[map all keyboard.map]

; A PC will start in Real Address Mode
	BITS 16

	section .text

	org 7c00h

; Video related constants

videoseg:	equ 0b800h
linesize:	equ 2*80
screensize:	equ 25*linesize
int15vec:	equ 4*15h

kbddata:	equ 60h 
kbdstatus:	equ 64h 

; Make sure that CS=0

start:
	cli
	jmp 0:.next
.next
	xor ax,ax
	mov ds,ax
	mov ss,ax
	mov sp,8000h
	sti
	call vidinit
	call int16
	call int15
	call i8042
	jmp start

; Read keyboard through BIOS Int 16h

int16:
	mov bx,txInt16
	call sout
.loop:
	mov ah,00h	; 00h original, 10h extended
	int 16h
	mov dl,al
	mov bx,txAscii
	call sout
	call hexout
	mov bx,txScancode
	call sout
	shr ax,8
	call hexout
	cmp dl,20h
	jne .loop
	ret

; Trap INT 15 and display scancodes passed through function AH=15h
int15:
	call nl
	mov bx,txTrapInt15
	call sout
	cli
	mov eax,[int15vec]
	mov [int15_orig],eax
	mov word [int15vec],int15trap
	mov word [int15vec+2],0
	xor ax,ax
	mov [press_count],ax
	sti
.waitloop:
	mov ax,[int15vec]
	cmp ax,int15trap
	je .waitloop
	ret

; Read directly from 8042. Disables interrupts, so INT9 gets not called.
i8042:
	cli
	call nl
	mov bx,tx8042
	call sout
.waitloop2
	in al,kbdstatus
	test al,1
	jz .waitloop2
	in al,kbddata
	mov bx,txData
	call sout
	call hexout
	jmp .waitloop2
	ret

int15trap:
	cmp ah,4fh
	jne .skip
	push edx
	mov bx,txScancode
	call sout
	call hexout
	test al,80h
	jne .release
	inc word [press_count]
	jmp .inspect
.release:
	dec word [press_count]
.inspect:
	push bx
	mov bx,[press_count]
	cmp bx,10
	jle .continue
	; Reset int 15 vector
	cli
	push eax
	mov eax,[int15_orig]
	mov [int15vec],eax
	pop eax
	sti
.continue:
	pop bx
	pop edx
.skip:
	jmp far [int15_orig]

resetcursor:
	push ax
	xor ax,ax
	mov [charptr],ax
	pop ax
	ret
	
vidinit:
	push ax
	mov ax,videoseg
	mov [charptrseg],ax
	call resetcursor
	call clearscreen
	pop ax
	ret

clearscreen:
	push es
	push eax
	push ecx
	push edi
	mov es,[charptrseg]
	xor eax,eax
	cld	; increment count
	mov edi,0
	mov ecx,screensize/4
	es rep stosd
	pop edi
	pop ecx
	pop eax
	pop es
	ret
	
hexdword:
	push eax
	shr eax,16
	call hexword
	pop eax
; fallthrough hexword
hexword:
	push ax
	shr ax,8
	call hexout
	pop ax
; fallthrough hexout
hexout:
	push ax
	shr al,4
	call nibout
	pop ax
; fallthrough nibout
nibout:
	push ax
	and al,0fh
	cmp al,9
	jle .nibout1
	add al,7
.nibout1:
	add al,30h
	call charout
	pop ax
	ret
; fallthrough charout
charout:
	push es
	push bx
	les bx,[charptr]
	cmp bx,screensize
	jl .noscroll
	call scroll
	mov bx,screensize-linesize
.noscroll:
	mov [es:bx],al
	inc bx
	mov byte [es:bx],0fh
	inc bx
	mov [charptr],bx
	pop bx
	pop es
	ret
; Scroll just moves the screen content. It does not change the charptr

scroll:
	push es
	push esi
	push edi
	push ecx
	mov es,[charptrseg]
	cld	; increment count
	mov edi,0
	mov esi,linesize
	mov ecx,screensize/4
	es rep movsd
	pop ecx
	pop edi
	pop esi
	pop es
	ret

; Go to next line.

nl:
	push ax
	push bx
	mov ax,[charptr]
	mov bl,2*80
	div bl
; Now AL contains quotient and AH contains reminder. If it is 0 there is no need
; to start a new line.
	cmp ah,0
	jz .nonl
	xor ah,ah
	inc al
	mul bl
	cmp ax,screensize
	jl .noscroll
	call scroll
	mov ax,screensize-linesize
.noscroll:
	mov [charptr],ax
.nonl:
	pop bx
	pop ax
	ret

; Output characters till 00h
sout:
	push ax
	push bx
.loop:
	mov al,[bx]
	cmp al,0
	je .end
	call charout
	inc bx
	jmp .loop
.end:
	pop bx
	pop ax
	ret

; Guard the area containing boot signature.

%if ($-$$)>1feh
%error Too large to fit in one disk block
%endif

	times 510-($-$$) db 0
	db 55h
	db 0AAh

;	times 2048-($-$$) db 0
	
	section .data align=4

txInt16:
	db "Int16 with AH=00h:",0
txAscii:
	db " Ascii=",0
txScancode:
	db " Scancode=",0
txTrapInt15:
	db "Trap scancodes passed to Int15 with AH=4F:",0
tx8042:
	db "Direct read from 8042:",0
txData:
	db " data=",0

	section .bss start=0600h

int15_orig:
	resb 4
charptr:
	resb 2
charptrseg:
	resb 2
press_count:
	resb 2
